#' ---
#' title: "Multiple RSF's with Fisher data"
#' author: "John Fieberg & Johannes Signer"
#' date: ""
#' output:
#'  html_document:
#'    toc: yes
#'    toc_float: true
#' ---

#' **Purpose**:  demonstrate methods for analyzing data from multiple animals, allowing for multiple random slopes. 
#' 
#' **NOTE** the mixed models here take considerable time to run (you may want to use caching to speed up if you plan to run multiple times). Change the following line, below, to enable caching:
#' opts_chunk$set(fig.width=12,fig.height=4.5, error=TRUE,cache = FALSE).
#' 
#' #### Preamble
#' 
#' Load libraries
#+warning=FALSE, message=FALSE
library(knitr)
library(lubridate)
library(raster)
library(move)
library(amt) 
library(broom)
library(nlme)
library(lme4)
library(tidyverse)
library(tictoc)
library(here)
library(glmmTMB)
library(GGally)

options(width=165)
opts_chunk$set(fig.width=12,fig.height=4.5, error=TRUE,cache = TRUE)


#' Record time for running all code
tic("total")


#' ## RSF models
#' 
#' Read in annotated available data for RSF modeling
rsfdat <- read_rds("data/rsf_dat.rds")

#' Forest versus non-forest
rsfdat$forest <- ifelse(rsfdat$landC == "forest", 1, 0)

#' Weight available data 
rsfdat$w <- ifelse(rsfdat$case_ == 1, 1, 5e3)

#' ### Individual fits
#'
#' Lets start by fitting "full" models to individuals and then look to see if coeficients are correlated (high values of one coefficient correspond to high/low values of another coefficient)
class(rsfdat) <- class(rsfdat)[-1] # this should be fixed in the dev version
rsffits <- rsfdat %>% group_by(id) %>% nest %>% 
  mutate(mod = map(data, function(x) glm(case_ ~ elev + popD + forest, data = x, weight=w,family = binomial))) 


#' Now, summarize fits to individual animals
rsffits2 <- rsffits %>%
  dplyr::mutate(tidy = purrr::map(mod, broom::tidy),
                n = purrr::map(data, nrow) %>% simplify())
rsffits2 %>% unnest(tidy) %>% print(., n=1000)

#' Keep just the estimates for later comparisons
rsf_coefs<-rsffits2 %>% unnest(tidy) %>% 
  select(-(std.error:p.value))

rsf_coefs_wide <- rsf_coefs %>% spread(., term, estimate)

#' Explore coefficients using a pair plot
my_fn <- function(data, mapping, ...){
  p <- ggplot(data = data, mapping = mapping) + 
    geom_point() + 
    geom_smooth(method=lm, fill="blue", color="blue", ...)
  p
}

rsf_coefs_wide %>% select(elev, popD, forest) %>%    
   GGally::ggpairs(., lower = list(continuous = my_fn), upper = list(continuous = wrap("cor", method= "spearman")))

#' ### glmmTMB independent random slopes model
#' 
#' Process: 
#' 
#' 1. Set up model, but do not fit
#' 2. Set random intercept variance to large fixed value, set other variance components to 0
#' 3. Fit the model

#' THis is the model we'd like to fit (where coefficients may be correlated), but you will wait a long time. And, it requires too many variance parameters for the small number of individuals

tic("mixed mod")
fisher.tmp <- glmmTMB(case_ ~ elev + forest + popD + (1|id) + (0 + elev|id) + (0 + popD |id) +(0 +forest | id), family=binomial(), data = rsfdat,
                      doFit=FALSE, weights = w)

#' Set variance of random intercept to 10^6
fisher.tmp$parameters$theta[1] <- log(1e3)
nvarparm<-length(fisher.tmp$parameters$theta)
fisher.tmp$mapArg <- list(theta=factor(c(NA,1:(nvarparm-1))))
fisher.rsf.ind <- glmmTMB:::fitTMB(fisher.tmp)
summary(fisher.rsf.ind)
toc()

#' ### Comparisons
#' 
#' Compare fixed effects (above) to mean coefficient from individual fits. These are quite similar.
# MEAN COEFFICIENT FROM INDIVIDUAL FITS 
rsf_coefs %>% filter(term!="(Intercept)") %>% group_by(term) %>% summarize(mean=mean(estimate), se.mean=sd(estimate)/sqrt(n()))

# POPULATION MEAN COEFFICIENT FROM MIXED MODEL
summary(fisher.rsf.ind)$coef$cond[-1,]

#'Compare variance of individual esimtates to variance component from mixed model. Note, the variance of the individual coefficients will be larger (too big) since it includes true variability and sampling variability.

# VARIANCE OF COEFFICIENTS FROM INDIVIDUAL FITS
rsf_coefs %>% filter(term!="(Intercept)") %>% group_by(term) %>% summarize(var_elev=var(estimate), sd_elev=sd(estimate))

# VARIANCE COMPONENT FROM MIXED MODEL
summary(fisher.rsf.ind)$varcor

#' Look at individual coefficients
# COEFFICIENTS FROM INDIVIDUAL FITS
rsf_coefs2 <- rsf_coefs %>% filter(term!="(Intercept)") %>% arrange(term, id)
rsf_coefs2$method<-"indiv_fits"                       
names(rsf_coefs2)[4]<-"Estimate"
rsf_coefs2<-rsf_coefs2[,-2]

# COEFFICIENTS FROM MIXED MODEL
mixed_coefs <- coef(fisher.rsf.ind)$cond$id[,-1]
mixed_coefs$id <- rownames(mixed_coefs)
mixed_coefs <- mixed_coefs %>% 
  gather('elev', 'forest', 'popD', key="term", value="Estimate")
mixed_coefs$method<-"Mixed"

#' Combine and plot
allests<-rbind(mixed_coefs, rsf_coefs2)
 
#' Note that because of the large sample size, individual coefficients show very little shrinkage towards the overall mean.
#+fig.width=14, fig.height=6
ggplot(data=allests, 
       aes(x=id, y=Estimate, col=method))+
  geom_point(size=3.5, position=position_dodge(width=0.3))+
  xlab("")+ylab(expression(hat(beta)))+facet_wrap(~term)


#' ## SSF model

#' Read in annotated available data for SSF modeling
ssfdat <- read_rds("data/ssf_dat.rds")


#' Create ID for each unique strata, by combining ID and step_id
ssfdat$step_id = with(ssfdat, paste0(id, step_id_, sep = ""))

#'  Since not all animals experience
#' all habitat types, lets just explore forest versus non-forest
ssfdat$forest <- ifelse(ssfdat$landC == "forest", 1, 0)


#' ### glmmTMB: random slopes model
#' 
#' The process is the same as for the mixed RSF:
#' 
#' 1. Set up model, but do not fit
#' 2. Set random intercept variance to large fixed value, set other variance components to 0
#' 3. Fit the model
#'
#' However, there are a few differences:
#' 
#' - we will use a Poisson likelihood rather than logistic 
#' - we won't need weights
#' - we will include fixed intercepts for each step_id
#' 
tic("mixed ssf")
fisher.tmp <- glmmTMB(case_ ~ elev + popD + forest + (1|step_id) + (0 + elev|id) + (0 + popD|id) + (0+ forest |id), family=poisson(), data = ssfdat,
                      doFit=FALSE)

#' Set variance of random intercept to 10^6
fisher.tmp$parameters$theta[1] <- log(1e3)
nvarparm<-length(fisher.tmp$parameters$theta)
fisher.tmp$mapArg <- list(theta=factor(c(NA,1:(nvarparm-1))))
fisher.ssf <- glmmTMB:::fitTMB(fisher.tmp)
summary(fisher.ssf)
toc()

#' ### Using Ts.Estim
library(TwoStepCLogit)
tic("two-step")
twostep<-Ts.estim(formula = case_ ~  elev+ popD+forest+
           strata(step_id) +cluster(id),data = ssfdat, random = ~ elev+popD+forest, all.m.1=F) 
twostep
toc()

#' ### Individual fits 
class(ssfdat) <- class(ssfdat)[-1] # this should be fixed in the dev version
ssffits <- ssfdat %>% group_by(id) %>% nest %>% 
  mutate(mod = map(data, function(x) (fit_issf(case_ ~ elev + popD + forest + strata(step_id), data = x))))


#' Now, summarize fits to individual animals
ssffits <- ssffits %>%
  mutate(tidy = map(mod, ~ broom::tidy(.x$model)),
         n = map_int(data, nrow))

ssffits %>% unnest(tidy) %>% filter(term!="(Intercept)") %>% print(., n=1000)

#' Keep just coefficients for comparisons
ssf_coefs<-ssffits %>% unnest(tidy) %>% 
  select(-(std.error:conf.high)) 

#' ### Comparisons
#' 
#' Compare fixed effects from mixed model and TS.Estim (above) to mean coefficient from individual fits
# MEAN COEFFICIENT FROM INDIVIDUAL FITS 
se<-function(x){sd(x)/sqrt(length(x))}
ssf_coefs %>% filter(term!="(Intercept)") %>% group_by(term) %>%   summarize(mean=mean(estimate), se=se(estimate))

# POPULATION MEAN COEFFICIENT FROM MIXED MODEL
summary(fisher.ssf)$coef$cond[-1,]

# POPULATION MEAN COEFFICIENT FROM TWO-STEP (TS.Estim)
twostep$beta

#'Compare variance of individual esimtates to variance component from mixed model.
# VARIANCE OF COEFFICIENTs FROM INDIVIDUAL FITS
ssf_coefs %>% filter(term!="(Intercept)") %>% group_by(term) %>% summarize(var_elev=var(estimate), sd_elev=sd(estimate))

# VARIANCE OF COEFFICIENTs FROM MIXED MODEL
summary(fisher.ssf)$varcor #var

# VARIANCE OF COEFFICIENTs FROM TWO-STEP (TS.Estim)
sqrt(twostep$D) # sd

#' Look at individual coefficients
# COEFFICIENTs FROM INDIVIDUAL FITS
ssf_coefs2 <- ssf_coefs %>% filter(term!="(Intercept)") %>% arrange(term, id)
ssf_coefs2$method<-"indiv_fits"                       
names(ssf_coefs2)[4]<-"Estimate"
ssf_coefs2<-ssf_coefs2[,-2]

# COEFFICIENTs FROM MIXED MODEL
mixed_coefs <- coef(fisher.ssf)$cond$id[,-1]
mixed_coefs$id <- rownames(mixed_coefs)
mixed_coefs <- mixed_coefs %>% 
  gather('elev', 'forest', 'popD', key="term", value="Estimate")
mixed_coefs$method<-"Mixed"


# COEFFICIENTS FROM TWO-STEP (TS.Estim)
twostepcoefs<-matrix(twostep$beta, nrow(twostep$r.effect), ncol(twostep$r.effect), byrow=TRUE)+twostep$r.effect
twostepcoefs<-as.data.frame(twostepcoefs)
twostepcoefs$method<-"two_step"
twostepcoefs$id<-rownames(twostep$r.effect)
twostepcoefs<-twostepcoefs %>% gather('elev', 'popD','forest', key="term", value="Estimate")

#' Combine and plot
allests<-rbind(mixed_coefs, ssf_coefs2,twostepcoefs)

#' Note here that a few coefficients fo relevation and popD show considerable shrinkage. These are coefficients with large SEs as seen by inspecting the individual fits.
#+fig.width=14, fig.height=6
ggplot(data=allests, 
       aes(x=id, y=Estimate, col=method))+
  geom_point(size=3.5, position=position_dodge(width=0.3))+
  xlab("")+ylab(expression(hat(beta)))+facet_wrap(~term)

#' Look at elevation, with coefficients sorted by SE. Note, the 3 coefficients with the largest SEs show the most shrinkage.
ssffits %>% unnest(tidy)  %>% filter(term=="elev") %>% arrange(desc(std.error)) %>% select(-c(statistic:conf.high))

#' Similar story for popD. M5 shows considerably shrinkage due to the large SE associated with its coefficient.
ssffits %>% unnest(tidy)  %>% filter(term=="popD") %>% arrange(desc(std.error)) %>% select(-c(statistic:conf.high))

#' Total Elapsed time      
toc()


#' ## Document Footer	
   
#' Session Information:	
#' 	
sessionInfo()	  
   