#' ---
#' title: "Simulate space use with Fisher data"
#' author: "John Fieberg & Johannes Signer"
#' date: ""
#' output:
#'  html_document:
#'    toc: yes
#'    toc_float: true
#' ---

#' **Purpose**:  demonstrate methods for simulating space use from fitted RSF and SSFs. 
#'

#' #### Preamble
#' 
#' Load libraries
#+warning=FALSE, message=FALSE
library(knitr)
library(lubridate)
library(raster)
library(amt) 
library(broom)
library(tidyverse)
library(tictoc)
library(here)
library(glmmTMB)

options(width=165)
opts_chunk$set(fig.width=12,fig.height=4.5, error=TRUE,cache = FALSE)


#' Record time for running all code
tic("total")


#' ## RSF models
#' 
#' Read in annotated available data for RSF modeling
rsfdat <- read_rds("data/rsf_dat.rds")

#' Forest versus non-forest
rsfdat$forest <- ifelse(rsfdat$landC == "forest", 1, 0)

#' Weight available data 
rsfdat$w <- ifelse(rsfdat$case_ == 1, 1, 5e3)

#' We will use only one animal to illustrate simulations. 
#' Simulations could be extendend to multiple animals, but not all the details
#' are worked out/implemented yet.
#'
#' Fit an RSF for F1
dat <- rsfdat %>% filter(id == "F1")
rsffit <- dat %>% mutate(elev = elev / 1e3) %>% 
  glm(case_ ~ forest + elev, data = ., weight = w, family = binomial())

summary(rsffit)

#' Next we prepare the environmental covariates: forest and elevation. 
landuse <- raster("data/landuse/landuse_study_area.tif") 
landuse <- projectRaster(landuse, crs = sp::CRS("+init=epsg:5070"), method = "ngb")
forest <- landuse %in% 41:43

elev <- raster("data/elevation/ASTER ASTGTM2 Elevation-20100101120000000-0-0.tif")
#' We have to resample the elevation raster, so that it matches the forests.
elev <- projectRaster(elev, crs = sp::CRS("+init=epsg:5070"))
elev <- resample(elev, landuse)
elev <- elev / 1e3

#' Next, we crop the raster to the 1 km buffer of a bounding box of the observed data. 

forest_small <- crop(forest, extent(c(range(dat$x_) + c(-1e3, 1e3), 
                                       range(dat$y_) + c(-1e3, 1e3))))

elev_small <- crop(elev, extent(c(range(dat$x_) + c(-1e3, 1e3), 
                                    range(dat$y_) + c(-1e3, 1e3))))

plot(forest_small)
plot(elev_small)

hull_polygon <- make_track(dat, x_, y_) %>% hr_mcp(level = 1) %>% hr_isopleths()

# mask the elev and forest rasters
elev_mcp <- mask(elev_small, hull_polygon)
forest_mcp<-mask(forest_small, hull_polygon)
  
#' Now we can calcualte UD from the RSF. Note, we are ignoring the intercept since it cancels out.
ud_lp <- coef(rsffit)[2] * forest_small + 
  coef(rsffit)[3] * elev_small 
ud <- exp(ud_lp)
ud <- ud / sum(ud[], na.rm = TRUE)
plot(ud)
with(filter(dat, case_ == 1), points(x_, y_, pch="."))
#' Better to plot the log of the UD. 
plot(log(ud))
with(filter(dat, case_ == 1), points(x_, y_, pch="."))

#' Now, if limit just to mcp
ud_lp <- coef(rsffit)[2] * forest_mcp + 
  coef(rsffit)[3] * elev_mcp 
ud <- exp(ud_lp)
ud <- ud / sum(ud[], na.rm = TRUE)
plot(ud)
with(filter(dat, case_ == 1), points(x_, y_, pch="."))
#' Better to plot the log of the UD. 
plot(log(ud))
with(filter(dat, case_ == 1), points(x_, y_, pch="."))

#' ## Simulate UDs from fitted SSF models

#' Read in annotated available data for SSF modeling
ssfdat <- read_rds("data/ssf_dat.rds")

#' Forest versus non-forest
ssfdat$forest <- ifelse(ssfdat$landC == "forest", 1, 0)

#' We will use only one animal to illustrate simulations. 
#' Simulations could be extendend to multiple animals, but not all the details
#' are worked out/implemented yet.

dat <- ssfdat %>% filter(id == "F1")
class(dat) <- c("stepx_xyt", "steps_xy", class(dat)) # class is lost due to the nesting. 
ssffit <- dat %>% 
  fit_issf(case_ ~ forest + strata(step_id_))

summary(ssffit)

landuse <- raster("data/landuse/landuse_study_area.tif") 
landuse <- projectRaster(landuse, crs = sp::CRS("+init=epsg:5070"), method = "ngb")
forest <- landuse %in% 41:43
names(forest) <- "forest"

# get movement characteristics
sl_char <- fit_sl_dist_base(filter(dat, case_ == 1) %>% pull(sl_))
sl_char$fit$estimate["shape"]

mk <- movement_kernel(
  shape = sl_char$fit$estimate["shape"], 
  scale = 1 / sl_char$fit$estimate["rate"], 
  template = forest
)
plot(mk)

hk <- habitat_kernel(
  coef = coef(ssffit), 
  resources = forest
)
  
plot(hk)

#' We want to restrict the area again to the area where the animal actually occured.
forest_small <- crop(forest, extent(c(range(dat$x1_) + c(-5e3, 5e3), 
                                      range(dat$y1_) + c(-5e3, 5e3))))
plot(forest_small)

hk_small <- habitat_kernel(
  coef = coef(ssffit), 
  resources = forest_small
)

#' Now lets simulate an UD
ud <- simulate_ud(mk, hk_small, start = as.numeric(dat[1, c("x1_", "y1_")]), 
                   n = 5e5)
plot(ud)


#' It is probably more realistic to add home ranging behaviour. We can do this by including
#' x and y and x^2 and y^2 in the model. 
y <- x <- hk_small
x[] <- xFromCell(hk_small, 1:ncell(hk_small))
y[] <- yFromCell(hk_small, 1:ncell(hk_small))

x <- (x - mean(x[])) / 1e3
y <- (y - mean(y[])) / 1e3

plot(x)
plot(y)

xy <- stack(x, y, x^2, y^2)
names(xy) <- c("x", "y", "x2", "y2")
plot(xy)

dat <- dat %>% extract_covariates(xy)

ssffit1 <- dat %>% 
  fit_issf(case_ ~ forest + x + y + x2 + y2 + strata(step_id_))

summary(ssffit1)

hk_small <- habitat_kernel(
  coef = coef(ssffit1), 
  resources = stack(forest_small, xy)
)

ud1 <- simulate_ud(mk, hk_small, start = as.numeric(dat[1, c("x1_", "y1_")]), 
                   n = 1e6)

plot(ud1)

#' Probably it is a good idea to increase n to 1e7 (this will take a bit longer for the simulations to run)
ud2 <- simulate_ud(mk, hk_small, start = as.numeric(dat[1, c("x1_", "y1_")]), 
                   n = 1e7)

plot(ud2)
with(filter(dat, case_ == 1), points(x1_, y1_, pch="."))

#' ## Document Footer	

#' Session Information:	
#' 	
sessionInfo()	  
